aboutsummaryrefslogtreecommitdiffstats
path: root/drivers
diff options
context:
space:
mode:
authorinmarket <andrewh@inmarket.com.au>2014-03-14 07:39:38 +1000
committerinmarket <andrewh@inmarket.com.au>2014-03-14 07:39:38 +1000
commitad1d70d2957d95161c831e764c6f81c84c332458 (patch)
tree639cc66297de6fe98e26608003871fe83406879a /drivers
parentb82448c3e23663a25a423402343fbca78253b80c (diff)
parentea5a1b849df6e5085a92957ad387f9e653674415 (diff)
downloaduGFX-ad1d70d2957d95161c831e764c6f81c84c332458.tar.gz
uGFX-ad1d70d2957d95161c831e764c6f81c84c332458.tar.bz2
uGFX-ad1d70d2957d95161c831e764c6f81c84c332458.zip
Merge branch 'master' into gwin
Diffstat (limited to 'drivers')
-rw-r--r--drivers/gaudin/Win32/gaudin_lld.c178
-rw-r--r--drivers/gaudin/Win32/gaudin_lld.mk5
-rw-r--r--drivers/gaudin/Win32/gaudin_lld_config.h64
-rw-r--r--drivers/gaudin/Win32/readme.txt9
-rw-r--r--drivers/gaudin/gadc/gaudin_lld.mk5
-rw-r--r--drivers/gaudio/Win32/driver.mk6
-rw-r--r--drivers/gaudio/Win32/gaudio_play_config.h63
-rw-r--r--drivers/gaudio/Win32/gaudio_play_lld.c185
-rw-r--r--drivers/gaudio/Win32/gaudio_record_config.h63
-rw-r--r--drivers/gaudio/Win32/gaudio_record_lld.c187
-rw-r--r--drivers/gaudio/Win32/readme.txt10
-rw-r--r--drivers/gaudio/gadc/driver.mk5
-rw-r--r--drivers/gaudio/gadc/gaudio_record_board_template.h (renamed from drivers/gaudin/gadc/gaudin_lld_board_template.h)20
-rw-r--r--drivers/gaudio/gadc/gaudio_record_config.h (renamed from drivers/gaudin/gadc/gaudin_lld_config.h)30
-rw-r--r--drivers/gaudio/gadc/gaudio_record_lld.c (renamed from drivers/gaudin/gadc/gaudin_lld.c)20
-rw-r--r--drivers/gaudio/gadc/readme.txt6
-rw-r--r--drivers/gdisp/HX8347D/gdisp_lld_HX8347D.c2
-rw-r--r--drivers/gdisp/ILI9341/gdisp_lld_ILI9341.c2
-rw-r--r--drivers/multiple/Win32/driver.mk (renamed from drivers/multiple/Win32/gdisp_lld.mk)4
19 files changed, 564 insertions, 300 deletions
diff --git a/drivers/gaudin/Win32/gaudin_lld.c b/drivers/gaudin/Win32/gaudin_lld.c
deleted file mode 100644
index e798f4ef..00000000
--- a/drivers/gaudin/Win32/gaudin_lld.c
+++ /dev/null
@@ -1,178 +0,0 @@
-/*
- * This file is subject to the terms of the GFX License. If a copy of
- * the license was not distributed with this file, you can obtain one at:
- *
- * http://ugfx.org/license.html
- */
-
-/**
- * @file drivers/gaudin/Win32/gaudin_lld.c
- * @brief GAUDIN - Driver file for Win32.
- */
-
-#include "gfx.h"
-
-#if GFX_USE_GAUDIN
-
-/* Include the driver defines */
-#include "src/gaudin/driver.h"
-
-#undef Red
-#undef Green
-#undef Blue
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-#include <stdio.h>
-#include <mmsystem.h>
-
-static HWAVEIN ah;
-static volatile int nUsedBuffers;
-static WAVEHDR *pWaveHdrs;
-static HANDLE waveInThread;
-static DWORD threadID;
-
-/*
-static void PrintWaveErrorMsg(DWORD err, TCHAR * str)
-{
- #define BUFFERSIZE 128
- char buffer[BUFFERSIZE];
-
- fprintf(stderr, "GAUDIN: ERROR 0x%08X: %s\r\n", err, str);
- if (mciGetErrorString(err, &buffer[0], sizeof(buffer)))
- fprintf(stderr, "%s\r\n", &buffer[0]);
- else
- fprintf(stderr, "0x%08X returned!\r\n", err);
-}
-*/
-
-/**************************** waveInProc() *******************************
- * We don't use CALLBACK_FUNCTION because it is restricted to calling only
- * a few particular Windows functions, namely some of the time functions,
- * and a few of the Low Level MIDI API. If you violate this rule, your app can
- * hang inside of the callback). One of the Windows API that a callback can't
- * call is waveInAddBuffer() which is what we need to use whenever we receive a
- * MM_WIM_DATA. My callback would need to defer that job to another thread
- * anyway, so instead just use CALLBACK_THREAD here instead.
- *************************************************************************/
-
-DWORD WINAPI waveInProc(LPVOID arg) {
- MSG msg;
- bool_t isRecording;
- (void) arg;
-
- isRecording = FALSE;
- while (GetMessage(&msg, 0, 0, 0)) {
- switch (msg.message) {
- case MM_WIM_DATA:
- GAUDIN_ISR_CompleteI((audin_sample_t *)((WAVEHDR *)msg.lParam)->lpData, ((WAVEHDR *)msg.lParam)->dwBytesRecorded/sizeof(audin_sample_t));
-
- /* Are we still recording? */
- if (isRecording) {
- /* Yes. Now we need to requeue this buffer so the driver can use it for another block of audio
- * data. NOTE: We shouldn't need to waveInPrepareHeader() a WAVEHDR that has already been prepared once.
- * Note: We are assuming here that both the application can still access the buffer while
- * it is on the queue.
- */
- waveInAddBuffer(ah, (WAVEHDR *)msg.lParam, sizeof(WAVEHDR));
-
- } else {
- /* We aren't recording, so another WAVEHDR has been returned to us after recording has stopped.
- * When we get all of them back, DoneAll will be equal to how many WAVEHDRs we queued
- */
- nUsedBuffers--;
- waveInUnprepareHeader(ah, (WAVEHDR *)msg.lParam, sizeof(WAVEHDR));
- }
-
- break;
-
- case MM_WIM_OPEN:
- isRecording = TRUE;
- break;
-
- case MM_WIM_CLOSE:
- isRecording = FALSE;
- break;
- }
- }
- return 0;
-}
-
-/*===========================================================================*/
-/* External declarations. */
-/*===========================================================================*/
-
-void gaudin_lld_init(const gaudin_params *paud) {
-// uint16_t channel;
-// uint32_t frequency;
-// audin_sample_t *buffer;
-// size_t bufcount;
-// size_t samplesPerEvent;
- WAVEFORMATEX wfx;
- size_t spaceleft;
- audin_sample_t *p;
- WAVEHDR *phdr;
- size_t nBuffers;
- size_t sz;
-
- if (!waveInThread) {
- if (!(waveInThread = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)waveInProc, 0, 0, &threadID))) {
- fprintf(stderr, "GAUDIN: Can't create WAVE recording thread\n");
- return;
- }
- CloseHandle(waveInThread);
- }
-
- nUsedBuffers = 0;
-
- wfx.wFormatTag = WAVE_FORMAT_PCM;
- wfx.nChannels = paud->channel == GAUDIN_STEREO ? 2 : 1;
- wfx.nSamplesPerSec = paud->frequency;
- wfx.nBlockAlign = wfx.nChannels * sizeof(audin_sample_t);
- wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
- wfx.wBitsPerSample = sizeof(audin_sample_t) * 8;
- wfx.cbSize = 0;
-
- if (waveInOpen(&ah, WAVE_MAPPER, &wfx, (DWORD_PTR)threadID, (DWORD_PTR)paud, CALLBACK_THREAD)) {
- fprintf(stderr, "GAUDIN: Can't open WAVE recording device\n");
- return;
- }
-
- /* We need to allocate a wave header for each buffer */
- nBuffers = (paud->bufcount + paud->samplesPerEvent - 1) / paud->samplesPerEvent;
- if (!(pWaveHdrs = gfxAlloc(nBuffers * sizeof(WAVEHDR)))) {
- fprintf(stderr, "GAUDIN: Buffer header allocation failed\n");
- return;
- }
-
- /* Prepare each buffer and send to the wavein device */
- spaceleft = paud->bufcount;
- for(p = paud->buffer, phdr = pWaveHdrs, spaceleft = paud->bufcount; spaceleft; p += sz, phdr++, spaceleft -= sz) {
- sz = spaceleft > paud->samplesPerEvent ? paud->samplesPerEvent : spaceleft;
- phdr->dwBufferLength = sz * sizeof(audin_sample_t);
- phdr->lpData = (LPSTR)p;
- phdr->dwFlags = 0;
- if (!waveInPrepareHeader(ah, phdr, sizeof(WAVEHDR))
- && !waveInAddBuffer(ah, phdr, sizeof(WAVEHDR)))
- nUsedBuffers++;
- else
- fprintf(stderr, "GAUDIN: Buffer prepare failed\n");
- }
- if (!nUsedBuffers)
- fprintf(stderr, "GAUDIN: Failed to prepare any buffers\n");
-}
-
-void gadc_lld_start(void) {
- if (nUsedBuffers)
- waveInStart(ah);
-}
-
-void gadc_lld_stop(void) {
- waveInReset(ah);
- while(nUsedBuffers) Sleep(1);
- if (pWaveHdrs) {
- gfxFree(pWaveHdrs);
- pWaveHdrs = 0;
- }
-}
-
-#endif /* GFX_USE_GAUDIN */
diff --git a/drivers/gaudin/Win32/gaudin_lld.mk b/drivers/gaudin/Win32/gaudin_lld.mk
deleted file mode 100644
index 3a822971..00000000
--- a/drivers/gaudin/Win32/gaudin_lld.mk
+++ /dev/null
@@ -1,5 +0,0 @@
-# List the required driver.
-GFXSRC += $(GFXLIB)/drivers/gaudin/Win32/gaudin_lld.c
-
-# Required include directories
-GFXINC += $(GFXLIB)/drivers/gaudin/Win32
diff --git a/drivers/gaudin/Win32/gaudin_lld_config.h b/drivers/gaudin/Win32/gaudin_lld_config.h
deleted file mode 100644
index 804d2fc4..00000000
--- a/drivers/gaudin/Win32/gaudin_lld_config.h
+++ /dev/null
@@ -1,64 +0,0 @@
-/*
- * This file is subject to the terms of the GFX License. If a copy of
- * the license was not distributed with this file, you can obtain one at:
- *
- * http://ugfx.org/license.html
- */
-
-/**
- * @file drivers/multiple/Win32/gaudin_lld_config.h
- * @brief GAUDIN Driver config file.
- *
- * @addtogroup GAUDIN
- * @{
- */
-
-#ifndef GAUDIN_LLD_CONFIG_H
-#define GAUDIN_LLD_CONFIG_H
-
-#if GFX_USE_GAUDIN
-
-/*===========================================================================*/
-/* Driver hardware support. */
-/*===========================================================================*/
-
-/**
- * @brief The audio input sample type
- */
-//typedef uint8_t audin_sample_t;
-typedef int16_t audin_sample_t;
-
-/**
- * @brief The maximum sample frequency supported by this audio device
- */
-#define GAUDIN_MAX_SAMPLE_FREQUENCY 44100
-
-/**
- * @brief The number of bits in a sample
- */
-//#define GAUDIN_BITS_PER_SAMPLE 8
-#define GAUDIN_BITS_PER_SAMPLE 16
-
-/**
- * @brief The format of an audio sample
- */
-//#define GAUDIN_SAMPLE_FORMAT ARRAY_DATA_8BITUNSIGNED
-#define GAUDIN_SAMPLE_FORMAT ARRAY_DATA_16BITSIGNED
-
-/**
- * @brief The number of audio channels supported by this driver
- */
-#define GAUDIN_NUM_CHANNELS 2
-
-/**
- * @brief The list of audio channels and their uses
- * @{
- */
-#define GAUDIN_MONO 0
-#define GAUDIN_STEREO 1
-/** @} */
-
-#endif /* GFX_USE_GAUDIN */
-
-#endif /* GAUDIN_LLD_CONFIG_H */
-/** @} */
diff --git a/drivers/gaudin/Win32/readme.txt b/drivers/gaudin/Win32/readme.txt
deleted file mode 100644
index b921c5e5..00000000
--- a/drivers/gaudin/Win32/readme.txt
+++ /dev/null
@@ -1,9 +0,0 @@
-This driver uses the Win32 audio system to provide a GAUDIN channel.
-
-It supports 2 channels, Channel 0 being a mono channel and Channel 1 being a stereo channel.
-
-For stereo, the samples are interleaved. Remember to allocate enough space for two samples per
-sample period.
-
-This is a simple driver that makes no changes to the mixer so set up the audio mixer using
-the windows control panel audio mixer (obviously in record mode) before starting recording.
diff --git a/drivers/gaudin/gadc/gaudin_lld.mk b/drivers/gaudin/gadc/gaudin_lld.mk
deleted file mode 100644
index c969e80b..00000000
--- a/drivers/gaudin/gadc/gaudin_lld.mk
+++ /dev/null
@@ -1,5 +0,0 @@
-# List the required driver.
-GFXSRC += $(GFXLIB)/drivers/gaudin/gadc/gaudin_lld.c
-
-# Required include directories
-GFXINC += $(GFXLIB)/drivers/gaudin/gadc
diff --git a/drivers/gaudio/Win32/driver.mk b/drivers/gaudio/Win32/driver.mk
new file mode 100644
index 00000000..1ea338d5
--- /dev/null
+++ b/drivers/gaudio/Win32/driver.mk
@@ -0,0 +1,6 @@
+# List the required driver.
+GFXSRC += $(GFXLIB)/drivers/gaudio/Win32/gaudio_record_lld.c \
+ $(GFXLIB)/drivers/gaudio/Win32/gaudio_play_lld.c
+
+# Required include directories
+GFXINC += $(GFXLIB)/drivers/gaudio/Win32
diff --git a/drivers/gaudio/Win32/gaudio_play_config.h b/drivers/gaudio/Win32/gaudio_play_config.h
new file mode 100644
index 00000000..4013e91f
--- /dev/null
+++ b/drivers/gaudio/Win32/gaudio_play_config.h
@@ -0,0 +1,63 @@
+/*
+ * This file is subject to the terms of the GFX License. If a copy of
+ * the license was not distributed with this file, you can obtain one at:
+ *
+ * http://ugfx.org/license.html
+ */
+
+/**
+ * @file drivers/gaudio/Win32/gaudio_play_config.h
+ * @brief GAUDIO Play Driver config file.
+ *
+ * @addtogroup GAUDIO
+ * @{
+ */
+
+#ifndef GAUDIO_PLAY_CONFIG_H
+#define GAUDIO_PLAY_CONFIG_H
+
+#if GFX_USE_GAUDIO && GAUDIO_NEED_PLAY
+
+/*===========================================================================*/
+/* Driver hardware support. */
+/*===========================================================================*/
+
+/**
+ * @brief The maximum sample frequency supported by this audio device
+ */
+#define GAUDIO_PLAY_MAX_SAMPLE_FREQUENCY 44100
+
+/**
+ * @brief The number of audio formats supported by this driver
+ */
+#define GAUDIO_PLAY_NUM_FORMATS 2
+
+/**
+ * @brief The available audio sample formats in order of preference
+ */
+#define GAUDIO_PLAY_FORMAT1 ARRAY_DATA_16BITSIGNED
+#define GAUDIO_PLAY_FORMAT2 ARRAY_DATA_8BITUNSIGNED
+
+/**
+ * @brief The number of audio channels supported by this driver
+ */
+#define GAUDIO_PLAY_NUM_CHANNELS 2
+
+/**
+ * @brief Whether each channel is mono or stereo
+ */
+#define GAUDIO_PLAY_CHANNEL0_IS_STEREO FALSE
+#define GAUDIO_PLAY_CHANNEL1_IS_STEREO TRUE
+
+/**
+ * @brief The list of audio channel names and their uses
+ * @{
+ */
+#define GAUDIO_PLAY_MONO 0
+#define GAUDIO_PLAY_STEREO 1
+/** @} */
+
+#endif /* GFX_USE_GAUDIO && GAUDIO_NEED_PLAY */
+
+#endif /* GAUDIO_PLAY_CONFIG_H */
+/** @} */
diff --git a/drivers/gaudio/Win32/gaudio_play_lld.c b/drivers/gaudio/Win32/gaudio_play_lld.c
new file mode 100644
index 00000000..6b4f2dab
--- /dev/null
+++ b/drivers/gaudio/Win32/gaudio_play_lld.c
@@ -0,0 +1,185 @@
+/*
+ * This file is subject to the terms of the GFX License. If a copy of
+ * the license was not distributed with this file, you can obtain one at:
+ *
+ * http://ugfx.org/license.html
+ */
+
+/**
+ * @file drivers/gaudio/Win32/gaudio_play_lld.c
+ * @brief GAUDIO - Play Driver file for Win32.
+ */
+
+#include "gfx.h"
+
+#if GFX_USE_GAUDIO && GAUDIO_NEED_PLAY
+
+/* Include the driver defines */
+#include "src/gaudio/driver_play.h"
+
+#undef Red
+#undef Green
+#undef Blue
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <stdio.h>
+#include <mmsystem.h>
+
+#define MAX_WAVE_HEADERS 2 // Larger numbers enable more buffering which is good for ensuring
+ // there are no skips due to data not being available, however larger
+ // numbers of buffers also create higher latency.
+
+static HWAVEOUT ah = 0;
+static volatile int nQueuedBuffers;
+static bool_t isRunning;
+static WAVEHDR WaveHdrs[MAX_WAVE_HEADERS];
+static HANDLE waveThread;
+static DWORD threadID;
+
+/**************************** waveProc() *******************************
+ * We don't use CALLBACK_FUNCTION because it is restricted to calling only
+ * a few particular Windows functions, namely some of the time functions,
+ * and a few of the Low Level MIDI API. If you violate this rule, your app can
+ * hang inside of the callback). One of the Windows API that a callback can't
+ * call is waveOutUnPrepareBuffer() which is what we need to use whenever we receive a
+ * MM_WOM_DONE. My callback would need to defer that job to another thread
+ * anyway, so instead just use CALLBACK_THREAD here instead.
+ *************************************************************************/
+
+static bool_t senddata(WAVEHDR *pwh) {
+ GAudioData *paud;
+
+ // Get the next data block to send
+ gfxSystemLock();
+ paud = gaudioPlayGetDataBlockI();
+ if (!paud && !nQueuedBuffers)
+ gaudioPlayDoneI();
+ gfxSystemUnlock();
+ if (!paud)
+ return FALSE;
+
+ // Prepare the wave header for Windows
+ pwh->dwUser = (DWORD_PTR)paud;
+ pwh->lpData = (LPSTR)(paud+1); // The data is on the end of the structure
+ pwh->dwBufferLength = paud->len;
+ pwh->dwFlags = 0;
+ pwh->dwLoops = 0;
+ if (waveOutPrepareHeader(ah, pwh, sizeof(WAVEHDR))) {
+ fprintf(stderr, "GAUDIO: Failed to prepare a play buffer");
+ exit(-1);
+ }
+
+ // Send it to windows
+ if (waveOutWrite(ah, pwh, sizeof(WAVEHDR))) {
+ fprintf(stderr, "GAUDIO: Failed to write the play buffer");
+ exit(-1);
+ }
+
+ nQueuedBuffers++;
+ return TRUE;
+}
+
+static DWORD WINAPI waveProc(LPVOID arg) {
+ MSG msg;
+ WAVEHDR *pwh;
+ (void) arg;
+
+ while (GetMessage(&msg, 0, 0, 0)) {
+ switch (msg.message) {
+ case MM_WOM_DONE:
+ pwh = (WAVEHDR *)msg.lParam;
+
+ // Windows - Let go!
+ waveOutUnprepareHeader(ah, pwh, sizeof(WAVEHDR));
+
+ // Give the buffer back to the Audio Free List
+ gfxSystemLock();
+ gaudioPlayReleaseDataBlockI((GAudioData *)pwh->dwUser);
+ gfxSystemUnlock();
+ pwh->lpData = 0;
+ nQueuedBuffers--;
+
+ // Are we stopping?
+ if (!isRunning) {
+ // Have we finished yet?
+ if (!nQueuedBuffers) {
+ gfxSystemLock();
+ gaudioPlayDoneI();
+ gfxSystemUnlock();
+ }
+ break;
+ }
+
+ // Try and get a new block
+ senddata(pwh);
+ break;
+ }
+ }
+ return 0;
+}
+
+/*===========================================================================*/
+/* External declarations. */
+/*===========================================================================*/
+
+bool_t gaudio_play_lld_init(uint16_t channel, uint32_t frequency, ArrayDataFormat format) {
+ WAVEFORMATEX wfx;
+
+ if (format != ARRAY_DATA_8BITUNSIGNED && format != ARRAY_DATA_16BITSIGNED)
+ return FALSE;
+
+ if (!waveThread) {
+ if (!(waveThread = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)waveProc, 0, 0, &threadID))) {
+ fprintf(stderr, "GAUDIO: Can't create WAVE play-back thread\n");
+ exit(-1);
+ }
+ CloseHandle(waveThread);
+ }
+
+ wfx.wFormatTag = WAVE_FORMAT_PCM;
+ wfx.nChannels = channel == GAUDIO_PLAY_STEREO ? 2 : 1;
+ wfx.nSamplesPerSec = frequency;
+ wfx.nBlockAlign = wfx.nChannels * (format == ARRAY_DATA_8BITUNSIGNED ? 1 : 2);
+ wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
+ wfx.wBitsPerSample = (format == ARRAY_DATA_8BITUNSIGNED ? 8 : 16);
+ wfx.cbSize = 0;
+
+ if (ah) {
+ waveOutClose(ah);
+ ah = 0;
+ }
+ if (waveOutOpen(&ah, WAVE_MAPPER, &wfx, (DWORD_PTR)threadID, 0, CALLBACK_THREAD)) {
+ fprintf(stderr, "GAUDIO: Can't open WAVE play-back device\n");
+ exit(-1);
+ }
+
+ return TRUE;
+}
+
+bool_t gaudio_play_lld_set_volume(uint8_t vol) {
+ if (!ah)
+ return FALSE;
+ return waveOutSetVolume(ah, (((uint16_t)vol)<<8)|vol) != 0;
+}
+
+void gaudio_play_lld_start(void) {
+ WAVEHDR *pwh;
+
+ isRunning = TRUE;
+ while (nQueuedBuffers < MAX_WAVE_HEADERS) {
+ // Find the empty one - there will always be at least one.
+ for(pwh = WaveHdrs; pwh->lpData; pwh++);
+
+ // Grab the next audio block from the Audio Out Queue
+ if (!senddata(pwh))
+ break;
+ }
+}
+
+void gaudio_play_lld_stop(void) {
+ isRunning = FALSE;
+ waveOutReset(ah);
+ while(nQueuedBuffers) Sleep(1);
+}
+
+#endif /* GFX_USE_GAUDIO && GAUDIO_NEED_PLAY */
diff --git a/drivers/gaudio/Win32/gaudio_record_config.h b/drivers/gaudio/Win32/gaudio_record_config.h
new file mode 100644
index 00000000..4d952e1d
--- /dev/null
+++ b/drivers/gaudio/Win32/gaudio_record_config.h
@@ -0,0 +1,63 @@
+/*
+ * This file is subject to the terms of the GFX License. If a copy of
+ * the license was not distributed with this file, you can obtain one at:
+ *
+ * http://ugfx.org/license.html
+ */
+
+/**
+ * @file drivers/gaudio/Win32/gaudio_record_config.h
+ * @brief GAUDIO Record Driver config file.
+ *
+ * @addtogroup GAUDIO
+ * @{
+ */
+
+#ifndef GAUDIO_RECORD_CONFIG_H
+#define GAUDIO_RECORD_CONFIG_H
+
+#if GFX_USE_GAUDIO && GAUDIO_NEED_RECORD
+
+/*===========================================================================*/
+/* Driver hardware support. */
+/*===========================================================================*/
+
+/**
+ * @brief The maximum sample frequency supported by this audio device
+ */
+#define GAUDIO_RECORD_MAX_SAMPLE_FREQUENCY 44100
+
+/**
+ * @brief The number of audio formats supported by this driver
+ */
+#define GAUDIO_RECORD_NUM_FORMATS 2
+
+/**
+ * @brief The available audio sample formats in order of preference
+ */
+#define GAUDIO_RECORD_FORMAT1 ARRAY_DATA_16BITSIGNED
+#define GAUDIO_RECORD_FORMAT2 ARRAY_DATA_8BITUNSIGNED
+
+/**
+ * @brief The number of audio channels supported by this driver
+ */
+#define GAUDIO_RECORD_NUM_CHANNELS 2
+
+/**
+ * @brief Whether each channel is mono or stereo
+ */
+#define GAUDIO_RECORD_CHANNEL0_IS_STEREO FALSE
+#define GAUDIO_RECORD_CHANNEL1_IS_STEREO TRUE
+
+/**
+ * @brief The list of audio channels and their uses
+ * @{
+ */
+#define GAUDIO_RECORD_MONO 0
+#define GAUDIO_RECORD_STEREO 1
+/** @} */
+
+#endif /* GFX_USE_GAUDIO && GAUDIO_NEED_RECORD */
+
+#endif /* GAUDIO_RECORD_CONFIG_H */
+/** @} */
diff --git a/drivers/gaudio/Win32/gaudio_record_lld.c b/drivers/gaudio/Win32/gaudio_record_lld.c
new file mode 100644
index 00000000..259707e3
--- /dev/null
+++ b/drivers/gaudio/Win32/gaudio_record_lld.c
@@ -0,0 +1,187 @@
+/*
+ * This file is subject to the terms of the GFX License. If a copy of
+ * the license was not distributed with this file, you can obtain one at:
+ *
+ * http://ugfx.org/license.html
+ */
+
+/**
+ * @file drivers/gaudio/Win32/gaudio_record_lld.c
+ * @brief GAUDIO - Record Driver file for Win32.
+ */
+
+#include "gfx.h"
+
+#if GFX_USE_GAUDIO && GAUDIO_NEED_RECORD
+
+/* Include the driver defines */
+#include "src/gaudio/driver_record.h"
+
+#undef Red
+#undef Green
+#undef Blue
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <stdio.h>
+#include <mmsystem.h>
+
+#define MAX_WAVE_HEADERS 2 // Larger numbers enable more buffering which is good for ensuring
+ // there are no skips due to data not being available, however larger
+ // numbers of buffers chews buffers on the free-list.
+
+static HWAVEIN ah = 0;
+static volatile int nQueuedBuffers;
+static bool_t isRunning;
+static WAVEHDR WaveHdrs[MAX_WAVE_HEADERS];
+static HANDLE waveThread;
+static DWORD threadID;
+
+/**************************** waveProc() *******************************
+ * We don't use CALLBACK_FUNCTION because it is restricted to calling only
+ * a few particular Windows functions, namely some of the time functions,
+ * and a few of the Low Level MIDI API. If you violate this rule, your app can
+ * hang inside of the callback). One of the Windows API that a callback can't
+ * call is waveInAddBuffer() which is what we need to use whenever we receive a
+ * MM_WIM_DATA. My callback would need to defer that job to another thread
+ * anyway, so instead just use CALLBACK_THREAD here instead.
+ *************************************************************************/
+
+static bool_t getbuffer(WAVEHDR *pwh) {
+ GAudioData *paud;
+
+ // Get the next data block to send
+ gfxSystemLock();
+ paud = gaudioRecordGetFreeBlockI();
+ if (!paud && !nQueuedBuffers)
+ gaudioRecordDoneI();
+ gfxSystemUnlock();
+ if (!paud)
+ return FALSE;
+
+ // Prepare the wave header for Windows
+ pwh->dwUser = (DWORD_PTR)paud;
+ pwh->lpData = (LPSTR)(paud+1); // The data is on the end of the structure
+ pwh->dwBufferLength = paud->size;
+ pwh->dwFlags = 0;
+ if (waveInPrepareHeader(ah, pwh, sizeof(WAVEHDR))) {
+ fprintf(stderr, "GAUDIO: Failed to prepare a record buffer");
+ exit(-1);
+ }
+
+ // Send it to windows
+ if (waveInAddBuffer(ah, pwh, sizeof(WAVEHDR))) {
+ fprintf(stderr, "GAUDIO: Failed to add the record buffer");
+ exit(-1);
+ }
+
+ nQueuedBuffers++;
+ return TRUE;
+}
+
+static DWORD WINAPI waveProc(LPVOID arg) {
+ MSG msg;
+ WAVEHDR *pwh;
+ GAudioData *paud;
+ (void) arg;
+
+ while (GetMessage(&msg, 0, 0, 0)) {
+ switch (msg.message) {
+ case MM_WIM_DATA:
+ pwh = (WAVEHDR *)msg.lParam;
+
+ // Windows - Let go!
+ waveInUnprepareHeader(ah, pwh, sizeof(WAVEHDR));
+
+ // Save the buffer in the audio record list
+ paud = (GAudioData *)pwh->dwUser;
+ paud->len = pwh->dwBytesRecorded;
+ gfxSystemLock();
+ gaudioRecordSaveDataBlockI(paud);
+ gfxSystemUnlock();
+ pwh->lpData = 0;
+ nQueuedBuffers--;
+
+ // Are we stopping?
+ if (!isRunning) {
+ // Have we finished yet?
+ if (!nQueuedBuffers) {
+ gfxSystemLock();
+ gaudioRecordDoneI();
+ gfxSystemUnlock();
+ }
+ break;
+ }
+
+ // Try and get a new block
+ getbuffer(pwh);
+ break;
+ }
+ }
+ return 0;
+}
+
+/*===========================================================================*/
+/* External declarations. */
+/*===========================================================================*/
+
+bool_t gaudio_record_lld_init(uint16_t channel, uint32_t frequency, ArrayDataFormat format) {
+ WAVEFORMATEX wfx;
+
+ if (format != ARRAY_DATA_8BITUNSIGNED && format != ARRAY_DATA_16BITSIGNED)
+ return FALSE;
+
+ if (!waveThread) {
+ if (!(waveThread = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)waveProc, 0, 0, &threadID))) {
+ fprintf(stderr, "GAUDIO: Can't create WAVE recording thread\n");
+ exit(-1);
+ }
+ CloseHandle(waveThread);
+ }
+
+ wfx.wFormatTag = WAVE_FORMAT_PCM;
+ wfx.nChannels = channel == GAUDIO_RECORD_STEREO ? 2 : 1;
+ wfx.nSamplesPerSec = frequency;
+ wfx.nBlockAlign = wfx.nChannels * (format == ARRAY_DATA_8BITUNSIGNED ? 1 : 2);
+ wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
+ wfx.wBitsPerSample = (format == ARRAY_DATA_8BITUNSIGNED ? 8 : 16);
+ wfx.cbSize = 0;
+
+ if (ah) {
+ waveInClose(ah);
+ ah = 0;
+ }
+ if (waveInOpen(&ah, WAVE_MAPPER, &wfx, (DWORD_PTR)threadID, 0, CALLBACK_THREAD)) {
+ fprintf(stderr, "GAUDIN: Can't open WAVE recording device\n");
+ exit(-1);
+ }
+
+ return TRUE;
+}
+
+void gaudio_record_lld_start(void) {
+ WAVEHDR *pwh;
+
+ if (!ah)
+ return;
+
+ while (nQueuedBuffers < MAX_WAVE_HEADERS) {
+ // Find the empty one - there will always be at least one.
+ for(pwh = WaveHdrs; pwh->lpData; pwh++);
+
+ // Grab the next audio block from the free-list
+ if (!getbuffer(pwh))
+ break;
+ }
+ if (!isRunning) {
+ isRunning = TRUE;
+ waveInStart(ah);
+ }
+}
+
+void gaudio_record_lld_stop(void) {
+ isRunning = FALSE;
+ waveInReset(ah);
+ while(nQueuedBuffers) Sleep(1);
+}
+
+#endif /* GFX_USE_GAUDIO && GAUDIO_NEED_RECORD */
diff --git a/drivers/gaudio/Win32/readme.txt b/drivers/gaudio/Win32/readme.txt
new file mode 100644
index 00000000..83cb9c33
--- /dev/null
+++ b/drivers/gaudio/Win32/readme.txt
@@ -0,0 +1,10 @@
+This driver uses the Win32 audio system to provide GAUDIO play and record channels.
+
+For PLAY - It supports 2 channels, Channel 0 being a mono channel and Channel 1 being a stereo channel.
+For RECORD - It supports 2 channels, Channel 0 being a mono channel and Channel 1 being a stereo channel.
+
+For stereo, the samples are interleaved. Remember to allocate enough space for two samples per
+sample period.
+
+This is a simple driver that makes no changes to the mixer so set up the audio mixer using
+the windows control panel audio mixer before starting recording/playback.
diff --git a/drivers/gaudio/gadc/driver.mk b/drivers/gaudio/gadc/driver.mk
new file mode 100644
index 00000000..4d79da25
--- /dev/null
+++ b/drivers/gaudio/gadc/driver.mk
@@ -0,0 +1,5 @@
+# List the required driver.
+GFXSRC += $(GFXLIB)/drivers/gaudio/gadc/gaudio_record_lld.c
+
+# Required include directories
+GFXINC += $(GFXLIB)/drivers/gaudio/gadc
diff --git a/drivers/gaudin/gadc/gaudin_lld.c b/drivers/gaudio/gadc/gaudio_record_lld.c
index 972f3dcf..ee994dc1 100644
--- a/drivers/gaudin/gadc/gaudin_lld.c
+++ b/drivers/gaudio/gadc/gaudio_record_lld.c
@@ -6,10 +6,10 @@
*/
/**
- * @file drivers/gaudin/gadc/gaudin_lld.c
- * @brief GAUDIN - Driver file for using the cpu ADC (via GADC).
+ * @file drivers/gaudio/gadc/gaudio_record_lld.c
+ * @brief GAUDIO - Record Driver file for using the cpu ADC (via GADC).
*
- * @addtogroup GAUDIN
+ * @addtogroup GAUDIO
*
* @{
*/
@@ -18,20 +18,20 @@
* We are now implementing the driver - pull in our channel table
* from the board definitions.
*/
-#define GAUDIN_LLD_IMPLEMENTATION
+#define GAUDIO_RECORD_IMPLEMENTATION
#include "gfx.h"
-#if GFX_USE_GAUDIN
+#if GFX_USE_GAUDIO && GAUDIO_NEED_RECORD
/* Double check the GADC system is turned on */
#if !GFX_USE_GADC
- #error "GAUDIN - The GADC driver for GAUDIN requires GFX_USE_GADC to be TRUE"
+ #error "GAUDIO - The GADC driver for GAUDIO requires GFX_USE_GADC to be TRUE"
#endif
/* Include the driver defines */
-#include "src/gaudin/driver.h"
+#include "src/gaudio/driver_record.h"
/*===========================================================================*/
/* External declarations. */
@@ -50,13 +50,13 @@ void gaudin_lld_init(const gaudin_params *paud) {
*/
}
-void gadc_lld_start(void) {
+void gaudin_lld_start(void) {
gadcHighSpeedStart();
}
-void gadc_lld_stop(void) {
+void gaudin_lld_stop(void) {
gadcHighSpeedStop();
}
-#endif /* GFX_USE_GAUDIN */
+#endif /* GFX_USE_GAUDIO && GAUDIO_NEED_RECORD */
/** @} */
diff --git a/drivers/gaudio/gadc/readme.txt b/drivers/gaudio/gadc/readme.txt
new file mode 100644
index 00000000..47d3f191
--- /dev/null
+++ b/drivers/gaudio/gadc/readme.txt
@@ -0,0 +1,6 @@
+This driver uses the generic GADC driver to provide a GAUDIO recording device.
+
+It supports whatever high speed device channels that your GADC driver and board supports.
+
+For stereo, the samples are interleaved. Remember to allocate enough space for two samples per
+sample period.
diff --git a/drivers/gdisp/HX8347D/gdisp_lld_HX8347D.c b/drivers/gdisp/HX8347D/gdisp_lld_HX8347D.c
index 35492541..1b581d4d 100644
--- a/drivers/gdisp/HX8347D/gdisp_lld_HX8347D.c
+++ b/drivers/gdisp/HX8347D/gdisp_lld_HX8347D.c
@@ -41,7 +41,7 @@
/* Driver local functions. */
/*===========================================================================*/
-#include "../drivers/gdisp/HX8347D/HX8347D.h"
+#include "drivers/gdisp/HX8347D/HX8347D.h"
#define write_reg(g, reg, data) { write_index(g, reg); write_data(g, data); }
diff --git a/drivers/gdisp/ILI9341/gdisp_lld_ILI9341.c b/drivers/gdisp/ILI9341/gdisp_lld_ILI9341.c
index 134e3614..d6679714 100644
--- a/drivers/gdisp/ILI9341/gdisp_lld_ILI9341.c
+++ b/drivers/gdisp/ILI9341/gdisp_lld_ILI9341.c
@@ -47,7 +47,7 @@
#define GDISP_INITIAL_BACKLIGHT 100
#endif
-#include "../drivers/gdisp/ILI9341/ILI9341.h"
+#include "drivers/gdisp/ILI9341/ILI9341.h"
/*===========================================================================*/
/* Driver local functions. */
diff --git a/drivers/multiple/Win32/gdisp_lld.mk b/drivers/multiple/Win32/driver.mk
index f9242380..d9654dfe 100644
--- a/drivers/multiple/Win32/gdisp_lld.mk
+++ b/drivers/multiple/Win32/driver.mk
@@ -1,2 +1,2 @@
-GFXINC += $(GFXLIB)/drivers/multiple/Win32
-GFXSRC += $(GFXLIB)/drivers/multiple/Win32/gdisp_lld_Win32.c
+GFXINC += $(GFXLIB)/drivers/multiple/Win32
+GFXSRC += $(GFXLIB)/drivers/multiple/Win32/gdisp_lld_Win32.c
diff --git a/drivers/gaudin/gadc/gaudin_lld_board_template.h b/drivers/gaudio/gadc/gaudio_record_board_template.h
index 89cc0c12..26e87d88 100644
--- a/drivers/gaudin/gadc/gaudin_lld_board_template.h
+++ b/drivers/gaudio/gadc/gaudio_record_board_template.h
@@ -6,15 +6,15 @@
*/
/**
* @file drivers/gaudin/gadc/gaudin_lld_board_template.h
* @brief GAUDIN Driver board config board file
*
- * @addtogroup GAUDIN
+ * @addtogroup GAUDIO
* @{
*/
-#ifndef _GAUDIN_LLD_BOARD_H
-#define _GAUDIN_LLD_BOARD_H
+#ifndef _GAUDIO_RECORD_BOARD_H
+#define _GAUDIO_RECORD_BOARD_H
/*===========================================================================*/
/* Audio inputs on this board */
@@ -24,14 +24,14 @@
* @brief The number of audio channels supported by this driver
* @note This is an example
*/
-#define GAUDIN_NUM_CHANNELS 1
+#define GAUDIO_RECORD_NUM_CHANNELS 1
/**
* @brief The list of audio channels and their uses
* @note This is an example
* @{
*/
-#define GAUDIN_MICROPHONE 0
+#define GAUDIO_RECORD_MICROPHONE 0
/** @} */
/**
@@ -39,12 +39,12 @@
* @note This is an example
* @{
*/
-#ifdef GAUDIN_LLD_IMPLEMENTATION
- static uint32_t gaudin_lld_physdevs[GAUDIN_NUM_CHANNELS] = {
+#ifdef GAUDIO_RECORD_LLD_IMPLEMENTATION
+ static uint32_t gaudin_lld_physdevs[GAUDIO_RECORD_NUM_CHANNELS] = {
GADC_PHYSDEV_MICROPHONE,
};
#endif
/** @} */
-#endif /* _GAUDIN_LLD_BOARD_H */
+#endif /* _GAUDIO_RECORD_BOARD_H */
/** @} */
@@ -6,15 +6,15 @@
*/
/**
- * @file drivers/gaudin/gadc/gaudin_lld_board_template.h
- * @brief GAUDIN Driver board config board file
+ * @file drivers/gaudio/gadc/gaudio_record_board_template.h
+ * @brief GAUDIO Record Driver board config board file
*
diff --git a/drivers/gaudin/gadc/gaudin_lld_config.h b/drivers/gaudio/gadc/gaudio_record_config.h
index a9fd02ae..22d8750f 100644
--- a/drivers/gaudin/gadc/gaudin_lld_config.h
+++ b/drivers/gaudio/gadc/gaudio_record_config.h
@@ -6,54 +6,54 @@
*/
/**
* @file drivers/gaudin/gadc/gaudin_lld_config.h
* @brief GAUDIN Driver config file.
*
- * @addtogroup GAUDIN
+ * @addtogroup GAUDIO
* @{
*/
-#ifndef GAUDIN_LLD_CONFIG_H
-#define GAUDIN_LLD_CONFIG_H
+#ifndef GAUDIO_RECORD_CONFIG_H
+#define GAUDIO_RECORD_CONFIG_H
-#if GFX_USE_GAUDIN
+#if GFX_USE_GAUDIO && GAUDIO_NEED_RECORD
/*===========================================================================*/
/* Driver hardware support. */
/*===========================================================================*/
/**
* @brief The audio input sample type
* @details For this driver it matches the cpu sample type
*/
-typedef adcsample_t audin_sample_t;
+typedef adcsample_t audio_record_sample_t;
/**
* @brief The maximum sample frequency supported by this audio device
* @details For this driver it matches the GADC maximum high speed sample rate
*/
-#define GAUDIN_MAX_SAMPLE_FREQUENCY GADC_MAX_HIGH_SPEED_SAMPLERATE
+#define GAUDIO_RECORD_MAX_SAMPLE_FREQUENCY GADC_MAX_HIGH_SPEED_SAMPLERATE
/**
* @brief The number of bits in a sample
* @details For this driver it matches the cpu sample bits
*/
-#define GAUDIN_BITS_PER_SAMPLE GADC_BITS_PER_SAMPLE
+#define GAUDIO_RECORD_BITS_PER_SAMPLE GADC_BITS_PER_SAMPLE
/**
* @brief The format of an audio sample
* @details For this driver it matches the cpu sample format
*/
-#define GAUDIN_SAMPLE_FORMAT GADC_SAMPLE_FORMAT
+#define GAUDIO_RECORD_SAMPLE_FORMAT GADC_SAMPLE_FORMAT
/**
- * For the GAUDIN driver that uses GADC - all the remaining config definitions are specific
+ * For the GAUDIO driver that uses GADC - all the remaining config definitions are specific
* to the board.
*/
/* Include the user supplied board definitions */
-#include "gaudin_lld_board.h"
+#include "gaudio_record_board.h"
-#endif /* GFX_USE_GAUDIN */
+#endif /* GFX_USE_GAUDIO && GAUDIO_NEED_RECORD */
-#endif /* GAUDIN_LLD_CONFIG_H */
+#endif /* GAUDIO_RECORD_CONFIG_H */
/** @} */
@@ -6,54 +6,54 @@
*/
/**
- * @file drivers/gaudin/gadc/gaudin_lld_config.h
- * @brief GAUDIN Driver config file.
+ * @file drivers/gaudio/gadc/gaudio_record_config.h
+ * @brief GAUDIN Record Driver config file.
*
@@ -6,54 +6,54 @@
/*===========================================================================*/
/**
- * @brief The audio input sample type
+ * @brief The audio record sample type
* @details For this driver it matches the cpu sample type
*/